/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/

#include "MutateOnTree.h"

using namespace std;

extern ranlxs_state_t * vstate;	//global variable for random number generator 

/* constructor */
MutateOnTree::MutateOnTree(EvolutionTree * evt, RNAWithPairingEvolModel * m, RNAWithPairing * obj, bool _verbose){
	model = m;
	evtree = evt;
	evtree->setrootobj(obj);
	verbose = _verbose;
}


/* calls the evolutionary model to perform mutation along the edge entering a child.
   internal use only; called by mutate()*/
void MutateOnTree::_mutate_on_edge(RootedTreeNode * child,  bool verboseflag, int cx){

	RNAWithPairing * obj;
	assert(obj);
	RootedTreeNode * p = child->getParent();
	
	
	map<RootedTreeNode*, RNAWithPairing *>::iterator iter = (evtree->objlist).find(p);
	if( iter != evtree->objlist.end() ){
		 //*obj = *(iter->second);	//copy RNAWithPairing from parent to child.
		 obj = new RNAWithPairing(*(iter->second));		
	}else{
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : RootedTreeNode has no associated object for mutation"<<endl; 
		exit(1);
	}
	
	/* mutate RNAWithPairing associated with child */	
	/* notice here we use the parental population size Np */
	model->mutate_exevent(obj, child->getEl(), verboseflag, cx);
	
	//evtree->evhist[child] = evhist;
	
	evtree->objlist[child] = obj;
}


/* calls the evolutionary model to simulate evolution along the evolutionary tree */
void MutateOnTree::mutate(const char * seqfilename, const char * alignfilename, 
							 const char * pairingfilename, const char * labelfilename, 
							 const char * treenexusfilename, bool verboseflag){
	
	ofstream out1(seqfilename, ios::app);
	if(!out1){
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : file "<<seqfilename<<" can not be opened!"<<endl;
		exit(1);
	}			
	
	ofstream out2(alignfilename, ios::app);
	if(!out2){
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : file "<<alignfilename<<" can not be opened!"<<endl;
		exit(1);
	}			
	
	ofstream out3(pairingfilename, ios::app);
	if(!out3){
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : file "<<pairingfilename<<" can not be opened!"<<endl;
		exit(1);
	}	

	ofstream out4(labelfilename, ios::app);
	if(!out4){
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : file "<<labelfilename<<" can not be opened!"<<endl;
		exit(1);
	}
	
	ofstream out5(treenexusfilename, ios::app);
	if(!out5){
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : file "<<treenexusfilename<<" can not be opened!"<<endl;
		exit(1);
	}

	stack<RootedTreeNode *> st;
	
	RootedTreeNode * r = evtree->tree->getRoot();		
	int len = r->getChildren().size();
	for(int i = len - 1; i >=0 ; i--)
		st.push( r->getChildren()[i] );
	
	/* dump root information */
	map<RootedTreeNode*, RNAWithPairing *>::iterator iter = (evtree->objlist).find(r);
	if( iter != evtree->objlist.end() ){				
		out1<<iter->second->getSeqStr()<<"\n";
		vector<string> *c = iter->second->getC();
		for(unsigned int i=0; i<c->size(); i++)
			out2<<(*c)[i]<<" ";
			out2<<"\n";
			out3<<iter->second->getPairingStr()<<"\n";
			out4<<iter->first->getLabel()<<"\n";
			//out4<<iter->first->getSerial()<<"\n";
	}
	else{
		cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
			<<" : RootedTreeNode has no associated object for mutation"<<endl; 
		exit(1);
	}


	/* breadth-first traveral to simulate mutation */


	int cx = 0;
	while( st.size() ){
		RootedTreeNode * x = st.top();		
		st.pop();
		
		cx++;
		//cout<<"mutate on RootedTreeNode "<<x->getLabel()<<endl;
		cout<<"simulation started node : "<<cx<<endl;

		_mutate_on_edge(x, verboseflag, cx);
		
		int i;
		int lenx = x->getChildren().size();
		if(lenx !=0) {//corrected in version 2.5, used to be len!=0, wrong
			for(i = lenx -1 ; i >= 0; i--)
				st.push(x->getChildren()[i]);
		}

		//cout<<"write out for "<<x->getSerial();
		map<RootedTreeNode*, RNAWithPairing *>::iterator iter = (evtree->objlist).find(x);
		if( iter != evtree->objlist.end() ){				
			out1<<iter->second->getSeqStr()<<"\n";
			vector<string> *c = iter->second->getC();
			for(unsigned int i=0; i<c->size(); i++)
				out2<<(*c)[i]<<" ";
			out2<<"\n";
			out3<<iter->second->getPairingStr()<<"\n";
			out4<<iter->first->getLabel()<<"\n";
			//out4<<iter->first->getSerial()<<"\n";
		}else{
			cerr<<"Fatal error at file "<<__FILE__<<" line "<<__LINE__
				<<" : RootedTreeNode has no associated object for mutation"<<endl; 
			exit(1);
		}	

		vector<RootedTreeNode *> xc = x->getParent()->getChildren();//get x and its siblings
		stack<RootedTreeNode *>  stcpy = st;
		vector<RootedTreeNode *> rtn;
		rtn.reserve(stcpy.size());
		while(stcpy.size()){
			rtn.push_back(stcpy.top());
			stcpy.pop();
		}

		bool bl=true;
		for(unsigned int i=0; i<rtn.size(); i++)
			for(unsigned int j=0; j<xc.size(); j++)
				if(rtn[i]->getSerial() == xc[j]->getSerial())//if there is child in the stack, don't erase
					bl=false;			

		if(bl){			
			//cout<<"erase "<<x->getParent()->getSerial()<<"\t";
			//if(!x->getParent()->isLeaf())
			//	cout<<x->getParent()->getSerial()<<" is not leaf"<<endl;
			evtree->eraseObj(x->getParent());		
		}

		if(x->isLeaf())
			evtree->eraseObj(x);
	}	
	//cerr<<endl;
	out5<<evtree->tree->nexus();
	out1.close();
	out2.close();
	out3.close();
	out4.close();	
	out5.close();
}


/*these four functions are not used */
EvolutionTree * MutateOnTree::get_evtree() const{ return evtree; }
void MutateOnTree::set_evtree(EvolutionTree * e){ evtree = e; }	
RNAWithPairingEvolModel * MutateOnTree::get_evmodel() const{ return model; }
void MutateOnTree::set_evmodel(RNAWithPairingEvolModel * m){ model = m; }
