/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/

#include "Parameters.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <getopt.h>
#include <cmath>
#include <cstring>
#include <ctime>

using namespace std;

static const char *VERSION = "rnasim v1.0\nCopyright (C) 2008 University of Pennsylvania.";
static const char *optString = "t:r:N:s:a:k:e:o:A:B:d:E:pvh?";
static const struct option longOpts[] = {
	{ "tree",required_argument, NULL, 't' },
	{ "rna",required_argument, NULL, 'r' },
	{ "population-size",required_argument, NULL, 'N' },
	{ "tree-scaling",required_argument, NULL, 's' },
	{ "alpha",required_argument, NULL, 'a' },
	{ "kappa",required_argument, NULL, 'k' },
	{ "optimal-energy",required_argument, NULL, 'e' },
	{ "output-file-prefix",required_argument, NULL, 'o' },
	{ "gamma-shape",required_argument, NULL, 'A' },
	{ "gamma-rate",required_argument, NULL, 'B' },
	{ "gamma-maxweight", required_argument, NULL, 'c'},
	{ "pool-size", required_argument, NULL, 'd'},
	{ "global-DeltaE", required_argument, NULL, 'E'},

	{ "progress",no_argument, NULL, 'p' },
	{ "version", no_argument, NULL, 'v' },
	{ "help", no_argument, NULL, 'h' },
	{ "help", no_argument, NULL, '?' },
	{ NULL, no_argument, NULL, 0 }
};


/* constructors : initialized parameters*/
Parameters::Parameters(){
	set_default_parameters();
}


void Parameters::set_default_parameters(){
	N = 50000;
	alpha = 0.0001;
	bsf = 1.0;
	kappa = 20;
	E_opt = -0.1;
	gamma_shape = 1.0;
	gamma_rate = 1.0;
	gamma_maxweight= 1.0;
	distr_pool_size = 16;	
	verbose = false;
	global_DeltaE = 0.0;
	fixed_global_DeltaE = false;
	
	out_pfx = "rnasim.out";
	out_nuc = "rnasim.out.nuc";
	out_sec = "rnasim.out.sec";
	out_sid = "rnasim.out.sid";
	out_tre = "rnasim.out.tre";
	out_pos = "rnasim.out.pos";

}

void Parameters::display_version(){
	printf("%s\n", VERSION);
}

void Parameters::display_usage_short(){
	cerr<<"rnasim: missing arguments"<<endl;
	cerr<<"Try `rnasim --help' for more information."<<endl;
}

void Parameters::display_usage_long(){
	cerr<<endl<<"Usage: rnasim [OPTION] -t <tree-file> -r <RNA-file>"<<endl<<endl;

	cerr<<"Options require an argument"<<endl;
	cerr<<"\t"<<"-t, --tree                  : tree file in Newick format (mandatory)"<<endl;
	cerr<<"\t"<<"-r, --rna                   : RNA file in FASTA format with sequence and secondary structure (mandatory)"<<endl;
	cerr<<"\t"<<"-N, --population-size       : population size (default is 50000)"<<endl;
	cerr<<"\t"<<"-s, --branch-scaling        : tree branch length scaling factor (default is 1.0)"<<endl;
	cerr<<"\t"<<"-a, --alpha                 : energy regularization factor (default is 0.0001)"<<endl;
	cerr<<"\t"<<"-k, --kappa                 : ratio of substitution to indels (default is 20)"<<endl;
	cerr<<"\t"<<"-e, --optimal-energy        : optimal folding free energy (default is -0.1, i.e., 10% lower than the folding energy of root RNA)"<<endl;
	cerr<<"\t"<<"-o, --output-file-prefix    : prefix for output file name (default is 'out')"<<endl;
	cerr<<"\t"<<"-A, --gamma-shape           : shape parameter for gamma distribution (default is 1.0)"<<endl;
	cerr<<"\t"<<"-B, --gamma-rate            : rate parameter for gamma distribution (default is 1.0)"<<endl;
	cerr<<"\t"<<"-c, --gamma-maxweight       : maximal weight sampled from a gamma distribution (default is 1.0)"<<endl;
	cerr<<"\t"<<"-d, --pool-size             : number of RNAs used to constructed distribution of substitution rates (default is 16)"<<endl;
	cerr<<"\t"<<"-E, --global-DeltaE         : fixed global DeltaE for maximal free energy change between a mutant and its wild type (default is computed from root RNA)"<<endl;

	cerr<<endl<<"Options require no argument"<<endl;
	cerr<<"\t"<<"-p, --progress              : display simulation progress"<<endl;
	cerr<<"\t"<<"-v, --version               : display version information and exit"<<endl;
	cerr<<"\t"<<"-h, --help                  : display this help and exit"<<endl;
	cerr<<"\t"<<"-?, --help                  : display this help and exit"<<endl<<endl;

	cerr<<"Comments and bug reports to <sguo@mail.med.upenn.edu>"<<endl;

}

void Parameters::display_parameters(){

	cout<<"Simulation parameters:"<<endl;
	cout<<"input-file[tree]                : "<<in_tre<<endl;
	cout<<"input-file[RNA]                 : "<<in_rna<<endl<<endl;

	cout<<"out-file[sequence]              : "<<out_nuc<<endl;
	cout<<"out-file[sequence id]           : "<<out_sid<<endl;
	cout<<"out-file[nucleotide position]   : "<<out_pos<<endl;
	cout<<"out-file[2nd structure]         : "<<out_sec<<endl;
	cout<<"out-file[scaled tree]           : "<<out_tre<<endl<<endl;
	cout<<"population size                 : "<<N<<endl;
	cout<<"branch scaling                  : "<<bsf<<endl;
	cout<<"optimal energy                  : "<<E_opt<<endl;
	cout<<"alpha                           : "<<alpha<<endl;
	cout<<"kappa                           : "<<kappa<<endl;
	cout<<"gamma-shape                     : "<<gamma_shape<<endl;
	cout<<"gamma_rate                      : "<<gamma_rate<<endl;
	cout<<"gamma_maxweight                 : "<<gamma_maxweight<<endl;
	cout<<"pool-size                       : "<<distr_pool_size<<endl;
	cout<<"global-DeltaE                   : "<<global_DeltaE<<endl;
	
	if(verbose)
		cout<<"progress monitoring             : yes"<<endl;

	//cout<<"Starting time                   : "<<clock()<<endl;

}
/* validate and initialize parameters*/
bool Parameters::process_parameters(int _argc, char ** _argv){

	int opt = 0;
	int longIndex = 0;
	bool valid = true;

	/*check required parameters*/
	bool has_tree = false;
	bool has_rna  = false;

	opt = getopt_long(_argc, _argv, optString, longOpts, &longIndex );
	if(opt==-1){
		display_usage_short();
		return false;
	}
	while(opt != -1){
		switch(opt){
			case 't':
				in_tre = optarg;
				if(!validate_parameter('t', in_tre))
					valid = false;
				has_tree = true;
				break;
			case 'r':
				in_rna = optarg;
				if(!validate_parameter('r', in_rna))
					valid = false;
				has_rna = true;
				break;
			case 'N':
				N  = atof(optarg);
				if(!validate_parameter('N', N))
					valid = false;
				break;
			case 's':
				bsf = atof(optarg);
				if(!validate_parameter('s', bsf))
					valid = false;
				break;
			case 'a':
				alpha = atof(optarg);
				if(!validate_parameter('a', alpha))
					valid = false;
				break;
			case 'k':
				kappa = atof(optarg);
				if(!validate_parameter('k', kappa))
					valid = false;
				break;
			case 'e':
				E_opt = atof(optarg);
				if(!validate_parameter('e', E_opt))
					valid = false;
				break;
			case 'o':
				out_pfx = (optarg);
				if(!validate_parameter('o', optarg))
					valid = false;
				{
					out_nuc = optarg;
					out_nuc.append(".nuc");
					out_sec = optarg;
					out_sec.append(".sec");
					out_sid = optarg;
					out_sid.append(".sid");
					out_pos = optarg;
					out_pos.append(".pos");
					out_tre = optarg;
					out_tre.append(".tre");
			
				}
				break;
			case 'A':
				gamma_shape = atof(optarg);
				if(!validate_parameter('A', gamma_shape))
					valid = false;
				break;
			case 'B':
				gamma_rate = atof(optarg);
				if(!validate_parameter('B', gamma_rate))
					valid = false;
				break;
			case 'c':
				gamma_maxweight = atof(optarg);
				if(!validate_parameter('c', gamma_maxweight))
					valid = false;
				break;
			case 'd':
				distr_pool_size = atoi(optarg);
				if(!validate_parameter('d', distr_pool_size))
					valid = false;
				break;
			case 'E':
				fixed_global_DeltaE = true;
				global_DeltaE = atof(optarg);
				if(!validate_parameter('E', global_DeltaE))
					valid = false;
				break;
			case 'p':
				verbose = true;
				break;
			case 'v':
				display_version();
				exit(EXIT_SUCCESS);
				return true;
			case 'h':/* fall-through is intentional */
			case '?':
				display_usage_long();
				exit(EXIT_SUCCESS);
				return true;
			default:
				;
		}
		opt = getopt_long( _argc, _argv, optString, longOpts, &longIndex );
	}
	if(!has_tree || !has_rna){
		display_usage_short();
		return false;
	}
	return valid;
}


/* return false if parameter value is invalid
 return true if parameter value is valid*/
bool Parameters::validate_parameter(char par, char * value){

	switch(par){
		case 't':
		case 'r': /* fall-through is intentional */
			{
				ifstream inFile(value, ios::in);
				if(!inFile){
					cerr<<"file "<<value<<" can not be opened!"<<endl;
					return false;
				}
				inFile.close();
			}
			break;
		case 'o':
			{	/* check. if output file already exists, quit*/
				string out_nuc(value);
				out_nuc.append(".nuc");
				ifstream OUT_nuc(out_nuc.c_str(), ios::in);
				if(OUT_nuc){
					cerr<<"Output file "<<out_nuc<<" already exists, remove or rename it!"<<endl;
					return false;
				}
				OUT_nuc.close();

				string out_sec(value);
				out_sec.append(".sec");
				ifstream OUT_sec(out_sec.c_str(), ios::in);
				if(OUT_sec){
					cerr<<"Output file "<<out_sec<<" already exists, remove or rename it!"<<endl;
					return false;
				}

				OUT_sec.close();
				string out_sid(value);
				out_sid.append(".sid");
				ifstream OUT_sid(out_sid.c_str(), ios::in);
				if(OUT_sid){
					cerr<<"Output file "<<out_sid<<" already exists, remove or rename it!"<<endl;
					return false;
				}
				OUT_sid.close();

				string out_tre(value);
				out_tre.append(".tre");
				ifstream OUT_tre(out_tre.c_str(), ios::in);
				if(OUT_tre){
					cerr<<"Output file "<<out_tre<<" already exists, remove or rename it!"<<endl;
					return false;
				}
				OUT_tre.close();

				string out_pos(value);
				out_pos.append(".pos");
				ifstream OUT_pos(out_pos.c_str(), ios::in);
				if(OUT_pos){
					cerr<<"Output file "<<out_pos<<" already exists, remove or rename it!"<<endl;
					return false;
				}
				OUT_pos.close();
			}
			break;
		default:
			;
	}
	return true;
}

/* return false if parameter value is invalid
return true if parameter value is valid*/
bool Parameters::validate_parameter(char par, double value){

	switch(par){
		case 'N':
			if(value<100.0){
				cerr<<"Invalid parameter value: N must be greater than 100"<<endl;
				return false;
			}
			break;
		case 's':
			if(value<0.0){
				cerr<<"Invalid parameter value: s must be greater than 0"<<endl;
				return false;
			}
			break;
		case 'a':
			if(abs(value)>1.0){
				cerr<<"Invalid parameter value: a must be in [-1,1]"<<endl;
				return false;
			}
			break;
		case 'k':
			if(value<0.0){
				cerr<<"Invalid parameter value: k must be greater than 0"<<endl;
				return false;
			}
			break;
		case 'e':
			if(abs(value)>100.0){
				cerr<<"Invalid parameter value: e must be in [-100,100]"<<endl;
				return false;
			}
			break;
		case 'A':
			if(value<0.0){
				cerr<<"Invalid parameter value: A must be greater than 0"<<endl;
				return false;
			}
			break;
		case 'B':
			if(value<0.0){
				cerr<<"Invalid parameter value: B must be greater than 0"<<endl;
				return false;
			}
			break;
		case 'c':
			if(value<1.0 || value>10.0){
				cerr<<"Invalid parameter value: c must be in [1,10]"<<endl;
				return false;
			}
			break;
		case 'd':
			if(value<4 || value>128){
				cerr<<"Invalid parameter value: d must be integers in [4,128]"<<endl;
				return false;
			}
			break;
		case 'E':
			if(value>0){
				cerr<<"Invalid parameter value: E must be smaller than 0"<<endl;
				return false;
			}
			break;
		default:
			;
	}

	return true;
}


