/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/


#include "RNAWithPairingEvolModel.h"


/* use extern "C" to include RNAeval C code in C++ code */
extern "C"{
	#include "fold_vars.h"
	#include "fold.h"
	#include "utils.h"
}
extern "C" float energy_of_struct (const char * , const char *);



using namespace std;

/* Initialize static data member at file scope */
double RNAWithPairingEvolModel::deltaE = 1000000.0;
int RNAWithPairingEvolModel::NumMutatedRNA = 0;

/**************************************************************************
	constructor and private utility functions
**************************************************************************/
RNAWithPairingEvolModel::RNAWithPairingEvolModel(){

	obj = NULL;	
	E_0 = 0.0;
	N = 100000.0;
	mu = 0.000001;
	alpha = 0.0001;
	kappa = 0.025;
	gamma_maxweight = 10.0;
	verbose = false;
}

void RNAWithPairingEvolModel::setparameters(double _N, double _alpha, double _E_opt, double _kappa, int _distr_pool_size, double _gamma_maxweight, bool _verbose){
	N = _N;
	alpha = _alpha;
	E_opt = _E_opt;	
	kappa = _kappa;
	distr_pool_size = _distr_pool_size;
	gamma_maxweight = _gamma_maxweight;
	verbose = _verbose;
}



void RNAWithPairingEvolModel::setparameters(double _N, double _alpha, double _E_opt, double _kappa, int _distr_pool_size, double _gamma_maxweight, bool _verbose, bool _fixed_global_DeltaE, double _global_DeltaE){
	N = _N;
	alpha = _alpha;
	E_opt = _E_opt;	
	kappa = _kappa;
	distr_pool_size = _distr_pool_size;
	gamma_maxweight = _gamma_maxweight;
	verbose = _verbose;
	fixed_global_DeltaE = _fixed_global_DeltaE;
	global_DeltaE = _global_DeltaE;
}



int RNAWithPairingEvolModel::getIndelLength(){
	double r = gsl_rng_uniform(vstate);
	double sum = 0.0;
	for(int i=0; i<constIndelLimit; i++){
		sum += _indellength[i];
		if(r<sum)
			return i+1;	
	}
	return constIndelLimit;
}

/*	Insert one nucleotide into the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAeval, return energy change
	0<mutsite<p.size() */
double RNAWithPairingEvolModel::insertTest(int mutsite, int mutnuc) const{
	
	vector<int>	*tp = obj->getP();	//numerical representation of base pairing
	vector<int>	*ts = obj->getS();	//numerical representation of base pairing
	vector<int>   p = *tp; 
	vector<int>   s = *ts; 
	
	/*	inserted x is the (mutsite+1)'th element, i.e. x has index mutsite*/
	_insertToVector(s, mutsite, mutnuc);
	_insertToVector(p, mutsite, -1);
	for(int j = 0; j<p.size(); j++)
		if(p[j]>=mutsite)
			p[j]++;

	/* two nucleotides on one side next to mutsite is paired  case 1~4. bulge : case 5 */
	/*        ________
		_____/	
		__|_|
             \mt______ case 1~4 */
	int oppositesite=-1;
	int oppositenuc = -1;		
	int newpair = 0;
	
	while(mutsite>0 && mutsite<p.size()-1){
		newpair=0;
		int newmutsite = -1;	
		if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
			if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
				if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
					oppositesite = p[mutsite-1]+1;
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
					oppositesite = p[mutsite-1]-1;
			}
			newmutsite = mutsite+1;
		}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
			if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
				if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
					oppositesite = p[mutsite+1]+1;				
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
					oppositesite = p[mutsite+1]-1;			
			}		
			newmutsite = mutsite-1;
		}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
			int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
			int z = p[mutsite+1];					//--( x )--y--( z )---
			if(x-z == 2 || z-x == 2){
				oppositesite = _min(x,z)+1;
			}
		}
		
		if(oppositesite>0){
			oppositenuc = s[oppositesite];
			if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
				p[mutsite] = oppositesite;
				p[oppositesite]  = mutsite; 
				newpair++;				
			}			
		}
		oppositesite = -1;
		oppositenuc = -1;
		mutsite = newmutsite;
		if(newpair==0) break;		
	}
	
	
	//cout<<"InsertTest calls computeEnergy"<<endl;
	double te = computeEnergy(&p, &s);
	double ec = te - E_0;
	
	return ec;
}


/*	Insert one nucleotide into the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAEval*/
void RNAWithPairingEvolModel::insertCommit(int mutsite, int mutnuc){
	
	vector<int>		*tp = obj->getP();	//numerical representation of base pairing
	vector<int>		*ts = obj->getS();	//numerical representation of base pairing
	vector<double>	*tw = obj->getW();	//weight vector: each site is tagged with a weight
	vector<string>	*tc = obj->getC();
	vector<int>		&p = *tp;
	vector<int>		&s = *ts;			
	vector<double>	&w = *tw;
	vector<string>	&c = *tc;
	double weight = 0.0;	//weight of inserted nucleotide

	if(mutsite==0)
		weight=w[0];
	else if(mutsite==w.size()-1)
		weight=w[w.size()-1];
	else
		weight =  (w[mutsite-1] + w[mutsite]) * 0.5;	//inserted nucleotide has mean weight of its two neighboring nt
	
	/*	first inserted x as the (mutsite+1)'th element, i.e. x has index mutsite*/
	_insertToVector(w, mutsite, weight);
	_insertToVector(s, mutsite, mutnuc);
	_insertToVector(p, mutsite, -1);
	for(int j = 0; j<p.size(); j++)
		if(p[j]>=mutsite)
			p[j]++;
	
	if(mutsite==0){	//boundary condition			
		_insertToVector(c, 0, NumToStr(-1));
	}else{
		if(c[mutsite-1]=="-1" || c[mutsite]=="-1")
			_insertToVector(c, mutsite, NumToStr(-1));
		else{
			string newcode;
			newcode.reserve(1000);
			newcode.append(1, '(');
			newcode.append(c[mutsite-1]);
			newcode.append(1, ',');
			newcode.append(c[mutsite]);
			newcode.append(1, ')');
			_insertToVector(c, mutsite, newcode);
		}
	}	


	int oppositesite=-1;
	int oppositenuc = -1;		
	int newpair = 0;
	
	while(mutsite>0 && mutsite<p.size()-1){
		newpair=0;
		int newmutsite = -1;
		if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
			if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
				if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
					oppositesite = p[mutsite-1]+1;
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
					oppositesite = p[mutsite-1]-1;
			}
			newmutsite = mutsite+1;
		}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
			if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
				if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
					oppositesite = p[mutsite+1]+1;				
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
					oppositesite = p[mutsite+1]-1;			
			}
			newmutsite = mutsite-1;
		}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
			int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
			int z = p[mutsite+1];					//--( x )--y--( z )---
			if(x-z == 2 || z-x == 2){
				oppositesite = _min(x,z)+1;
			}		
		}
		
		if(oppositesite>0){
			oppositenuc = s[oppositesite];
			if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
				p[mutsite] = oppositesite;
				p[oppositesite]  = mutsite; 
				newpair++;
			}
		}
		oppositesite = -1;
		oppositenuc = -1;
		mutsite = newmutsite;
		if(newpair==0) break;
	}
	
	//double te = computeEnergy(&p, &s);

	//cout<<"In insertCommit, mutant has energy  " <<te<<endl;
}


/*	Insert n nucleotides into the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAeval, return energy change*/
double RNAWithPairingEvolModel::multiinsertTest(int mutsite, vector<int> mutnucs) const{
	
	vector<int>	*tp = obj->getP();	//numerical representation of base pairing
	vector<int>	*ts = obj->getS();	//numerical representation of base pairing
	vector<int>   p = *tp;
	vector<int>   s = *ts;
	int origmutsite = mutsite;
	
	for(int i=0; i<mutnucs.size(); i++){
		int mutnuc = mutnucs[i];
		mutsite = origmutsite;
		
		/*	inserted x is the (mutsite+1)'th element, i.e. x has index mutsite*/
		_insertToVector(s, mutsite, mutnuc);
		_insertToVector(p, mutsite, -1);
		for(int j = 0; j<p.size(); j++)
			if(p[j]>=mutsite)
				p[j]++;

		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
	
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
	}
	
	//cout<<"MultiInsertTest calls computeEnergy"<<endl;
	double te = computeEnergy(&p, &s);
	double ec = te - E_0;

	//cout<<"MultiinsertTest EC = "<<ec<<endl;
	return ec;
}

/*	Insert one nucleotide into the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAEval*/
void RNAWithPairingEvolModel::multiinsertCommit(int mutsite, vector<int> mutnucs){
	
	vector<int>		*tp = obj->getP();	//numerical representation of base pairing
	vector<int>		*ts = obj->getS();	//numerical representation of base pairing
	vector<double>	*tw = obj->getW();	//weight vector: each site is tagged with a weight
	vector<string>	*tc = obj->getC();
	vector<int>		&p = *tp;
	vector<int>		&s = *ts;			
	vector<double>	&w = *tw;
	vector<string>	&c = *tc;
	double weight = 0.0;	//weight of inserted nucleotide
	int origmutsite = mutsite;
		
	if(mutsite==0)
		weight=w[0];
	else if(mutsite==w.size()-1)
		weight=w[w.size()-1];
	else
		weight =  (w[mutsite-1] + w[mutsite]) * 0.5;	//inserted nucleotide has mean weight of its two neighboring nt
	
	for(int i=0; i<mutnucs.size(); i++){
		
		int mutnuc = mutnucs[i];
		mutsite = origmutsite;
		
		/*	first inserted x as the (mutsite+1)'th element, i.e. x has index mutsite*/
		_insertToVector(w, mutsite, weight);
		_insertToVector(s, mutsite, mutnuc);
		_insertToVector(p, mutsite, -1);
		for(int j = 0; j<p.size(); j++)
			if(p[j]>=mutsite)
				p[j]++;
		
		if(mutsite==0){	//boundary condition			
			_insertToVector(c, 0, NumToStr(-1));
		}else{
			if(c[mutsite-1]=="-1" || c[mutsite]=="-1")
				_insertToVector(c, mutsite, NumToStr(-1));
			else{
				string newcode;
				newcode.reserve(1000);
				newcode.append(1, '(');
				newcode.append(c[mutsite-1]);
				newcode.append(1, ',');
				newcode.append(c[mutsite]);
				newcode.append(1, ')');
				_insertToVector(c, mutsite, newcode);
			}
		}	
		
		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
	
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
		
	}
	
	
	//double te = computeEnergy(&p, &s);
	//double ec = te - E_0;

	//cout<<"MultiinsertCommit EC = "<<ec<<endl;
	//cout<<"In MultiinsertCommit, mutant has energy  " <<te<<endl;
}


/*	Delete the nucleotide from the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAEval 0<mutsite<p.size()*/
double RNAWithPairingEvolModel::deleteTest(int mutsite) const{
	
	vector<int>	*tp = obj->getP();	//numerical representation of base pairing
	vector<int>	*ts = obj->getS();	//numerical representation of base pairing
	vector<int>   p = *tp;
	vector<int>   s = *ts;

	if(p[mutsite]>=0)	p[p[mutsite]] = -1;	//dissociate nucleotide pair

	_removeFromVector(s, mutsite);
	_removeFromVector(p, mutsite);
	
	for(int j=0; j<p.size(); j++)
		if(p[j]>mutsite)	
			p[j]--;
	
	/* after delete the nucleotide, new pairs may form, this situtation is exactly 
		the same as insertTest, except that here mutsite is treated after deletion */
	int mutnuc = s[mutsite];
	int oppositesite=-1;
	int oppositenuc = -1;		
	int newpair = 0;
	while(mutsite>0 && mutsite<p.size()-1){
		newpair=0;
		int newmutsite = -1;	
		if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
			if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
				if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
					oppositesite = p[mutsite-1]+1;
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
					oppositesite = p[mutsite-1]-1;
			}
			newmutsite = mutsite+1;
		}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
			if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
				if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
					oppositesite = p[mutsite+1]+1;				
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
					oppositesite = p[mutsite+1]-1;			
			}		
			newmutsite = mutsite-1;
		}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
			int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
			int z = p[mutsite+1];					//--( x )--y--( z )---
			if(x-z == 2 || z-x == 2){
				oppositesite = _min(x,z)+1;
			}
		}
		
		if(oppositesite>0){
			oppositenuc = s[oppositesite];
			if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
				p[mutsite] = oppositesite;
				p[oppositesite]  = mutsite; 
				newpair++;				
			}			
		}
		oppositesite = -1;
		oppositenuc = -1;
		mutsite = newmutsite;
		if(newpair==0) break;		
	}
	
	
	//cout<<"DeleteTest calls computeEnergy"<<endl;
	double te = computeEnergy(&p, &s);
	double ec = te - E_0;
	
	//cout<<ec<<endl;

	return ec;
}

double RNAWithPairingEvolModel::multideleteTest(int mutsite, int indelsize)const{
	
	vector<int>	*tp = obj->getP();	//numerical representation of base pairing
	vector<int>	*ts = obj->getS();	//numerical representation of base pairing
	vector<int>   p = *tp;
	vector<int>   s = *ts;

	for(int i=0; i<indelsize; i++){	
		if(mutsite>=p.size())/* boundary condition */
			break;
		if(p[mutsite]>=0)	p[p[mutsite]] = -1;	//dissociate nucleotide pair
		_removeFromVector(s, mutsite);
		_removeFromVector(p, mutsite);
	
		for(int j=0; j<p.size(); j++)
			if(p[j]>mutsite)	
				p[j]--;
		
		int mutnuc = s[mutsite];
		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
	
	}
	
	//cout<<"MultiDeleteTest calls computeEnergy"<<endl;
	double te = computeEnergy(&p, &s);
	double ec = te - E_0;
	
	//cout<<"MultideleteTest EC = "<<ec<<endl;
	//cout<<"MultideleteTest"<<endl;
	
	return ec;
}

/*	Delete the nucleotide from the sequence at a fixed position: 
	only generate the mutated sequence and pairing to be used by RNAEval*/
void RNAWithPairingEvolModel::deleteCommit(int mutsite){
	
	vector<int>		*tp = obj->getP();	//numerical representation of base pairing
	vector<int>		*ts = obj->getS();	//numerical representation of base pairing
	vector<double>	*tw = obj->getW();	//weight vector: each site is tagged with a weight
	vector<string>	*tc = obj->getC();
	vector<int>		&p = *tp;
	vector<int>		&s = *ts;			
	vector<double>	&w = *tw;
	vector<string>	&c = *tc;

	if(p[mutsite]>=0)	p[p[mutsite]] = -1;	//dissociate nucleotide pair

	_removeFromVector(s, mutsite);
	_removeFromVector(p, mutsite);
	_removeFromVector(w, mutsite);		//remove mutsite-indexed element from weight vector
	_removeFromVector(c, mutsite);		//remove mutsite-indexed element from code vector

	for(int j=0; j<p.size(); j++)
		if(p[j]>mutsite)	
			p[j]--;

	/* after delete the nucleotide, new pairs may form, this situtation is exactly 
		the same as insertTest, except that here mutsite is treated after deletion */
	int mutnuc = s[mutsite];
	int oppositesite=-1;
	int oppositenuc = -1;		
	int newpair = 0;
	while(mutsite>0 && mutsite<p.size()-1){
		newpair=0;
		int newmutsite = -1;	
		if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
			if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
				if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
					oppositesite = p[mutsite-1]+1;
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
					oppositesite = p[mutsite-1]-1;
			}
			newmutsite = mutsite+1;
		}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
			if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
				if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
					oppositesite = p[mutsite+1]+1;				
				else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
					oppositesite = p[mutsite+1]-1;			
			}		
			newmutsite = mutsite-1;
		}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
			int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
			int z = p[mutsite+1];					//--( x )--y--( z )---
			if(x-z == 2 || z-x == 2){
				oppositesite = _min(x,z)+1;
			}
		}
		
		if(oppositesite>0){
			oppositenuc = s[oppositesite];
			if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
				p[mutsite] = oppositesite;
				p[oppositesite]  = mutsite; 
				newpair++;				
			}			
		}
		oppositesite = -1;
		oppositenuc = -1;
		mutsite = newmutsite;
		if(newpair==0) break;		
	}
	
	//double te = computeEnergy(&p, &s);

	//cout<<"In deleteCommit, mutant has energy  " <<te<<endl;
}


void RNAWithPairingEvolModel::multideleteCommit(int mutsite, int indelsize){
	
	vector<int>		*tp = obj->getP();	//numerical representation of base pairing
	vector<int>		*ts = obj->getS();	//numerical representation of base pairing
	vector<double>	*tw = obj->getW();	//weight vector: each site is tagged with a weight
	vector<string>	*tc = obj->getC();
	vector<int>		&p = *tp;
	vector<int>		&s = *ts;			
	vector<double>	&w = *tw;
	vector<string>	&c = *tc;

	for(int i=0;i<indelsize; i++){
	
		if(mutsite>=p.size())	/*boundary condition */
			break;

		if(p[mutsite]>=0)	p[p[mutsite]] = -1;	//dissociate nucleotide pair

		_removeFromVector(s, mutsite);
		_removeFromVector(p, mutsite);
		_removeFromVector(w, mutsite);		//remove mutsite-indexed element from weight vector
		_removeFromVector(c, mutsite);		//remove mutsite-indexed element from code vector

		for(unsigned int j=0; j<p.size(); j++)
			if(p[j]>mutsite)	
				p[j]--;
		
		
		int mutnuc = s[mutsite];
		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
	}
	
	//double te = computeEnergy(&p, &s);
	//double ec = te - E_0;

	//cout<<"In MultideleteCommit, mutant has energy  " <<te<<endl;
	//cout<<"MultideleteCommit EC = "<<ec<<endl;
}



/* substitue nucleotide at mutsite with mutnuc */
double RNAWithPairingEvolModel::substituteTest(int mutsite, int mutnuc) const {
	
	vector<int>	*tp = obj->getP();	//numerical representation of base pairing
	vector<int>	*ts = obj->getS();	//numerical representation of base pairing
	vector<int>   p = *tp;
	vector<int>   s = *ts;

	s[mutsite] = mutnuc;
	
	/*based on whether mutated nucleotide is paired */
	if( p[mutsite]>=0 ){//there was a pair, check if pairing still exists
		if(s[p[mutsite]] != _standardpairing[mutnuc][0] && s[p[mutsite]]  != _standardpairing[mutnuc][1]){
			p[p[mutsite]] = -1;
			p[mutsite] = -1;		
		}
	}else{
		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
	}
	
		
	double te = computeEnergy(&p, &s);
	double ec = te - E_0;

	//cout<<"SubstituteTest EC = "<<ec<<endl;
	//cout<<"SubstituteTest"<<endl;
	
	return ec;
}


/* substitue nucleotide at mutsite with mutnuc */
void RNAWithPairingEvolModel::substituteCommit(int mutsite, int mutnuc){
	
	vector<int>   *  tp = obj->getP();	//numerical representation of base pairing
	vector<int>	  *  ts = obj->getS();	//numerical representation of base pairing
	vector<int>		&p = *tp;
	vector<int>		&s = *ts;			
	
	s[mutsite] = mutnuc;
	
	/*based on whether mutated nucleotide is paired */
	if( p[mutsite]>=0 ){//there was a pair, check if pairing still exists
		if(s[p[mutsite]] != _standardpairing[mutnuc][0] && s[p[mutsite]]  != _standardpairing[mutnuc][1]){
			p[p[mutsite]] = -1;
			p[mutsite] = -1;		
		}
	}else{
		int oppositesite=-1;
		int oppositenuc = -1;		
		int newpair = 0;
		while(mutsite>0 && mutsite<p.size()-1){
			newpair=0;
			int newmutsite = -1;	
			if(p[mutsite-1]>0 && p[mutsite+1]<0){//boundary condition
				if(p[mutsite-1]>0 && p[mutsite-1]<p.size()-1){
					if(p[mutsite-1]-1>=0 && p[mutsite-1]+1<0)//case 1 
						oppositesite = p[mutsite-1]+1;
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0)//case 2
						oppositesite = p[mutsite-1]-1;
				}
				newmutsite = mutsite+1;
			}else if(p[mutsite+1]>0 && p[mutsite-1]<0){// boundary condition
				if(p[mutsite+1] >0 && p[mutsite-1] <p.size()-1){
					if(p[mutsite+1]-1>=0 && p[mutsite+1]+1<0) //case 3
						oppositesite = p[mutsite+1]+1;				
					else if(p[mutsite-1]+1>=0 && p[mutsite-1]-1<0) //case 4
						oppositesite = p[mutsite+1]-1;			
				}		
				newmutsite = mutsite-1;
			}else if(p[mutsite-1]>=0 && p[mutsite+1]>=0){//neighboring nucleotides are paired 
				int x = p[mutsite-1];					//--(i-1)--i--(i+1)---
				int z = p[mutsite+1];					//--( x )--y--( z )---
				if(x-z == 2 || z-x == 2){
					oppositesite = _min(x,z)+1;
				}
			}
		
			if(oppositesite>0){
				oppositenuc = s[oppositesite];
				if( _standardpairing[oppositenuc][0] == mutnuc || _standardpairing[oppositenuc][1] == mutnuc ){
					p[mutsite] = oppositesite;
					p[oppositesite]  = mutsite; 
					newpair++;				
				}			
			}
			oppositesite = -1;
			oppositenuc = -1;
			mutsite = newmutsite;
			if(newpair==0) break;		
		}
	}
	
	//double te = computeEnergy(&p, &s);
	//double ec = te - E_0;

	//cout<<"SubstituteCommit EC = "<<ec<<endl;
}


/**************************************************************************
	sequence manipulations
**************************************************************************/
/*	-get paired sites and unpaired sites; paired site has nonzero value, unpaired site has value -1
	see RNAWithPairing for more information on numerical representation of base pairing
*/
double RNAWithPairingEvolModel::computeEnergy(const vector<int> * p, const vector<int> *s) const {
	
	int i = 0;
	string seq, pair;
	pair.reserve(p->size());
	seq.reserve(s->size());
	
	if(p->size()!=s->size()){
		cerr<<"Error : at at file "<<__FILE__<<" line "<<__LINE__
			<<"In RNAWithPairingEvolModel::computeEnergy(): unequal size of sequence and pairing"<<endl;
		exit(1);
	}
	for(i=0; i<s->size(); i++){
		switch(s->at(i)){
			case 0:
				seq.append(1, 'A');
				break;
			case 1:
				seq.append(1, 'U');
				break;
			case 2:
				seq.append(1, 'C');
				break;
			case 3:
				seq.append(1, 'G');
				break;
			default:
				seq.append(1, 'N');
				cerr<<"Error : at at file "<<__FILE__<<" line "<<__LINE__
					<<"In RNAWithPairingEvolModel::computeEnergy(): unknown nucleotide "<<s->at(i)<<" found "<<endl;
				exit(1);
		}
		
	}

	for(i=0; i<p->size(); i++){
		if(p->at(i) < 0)
			pair.append(1, '.');
		else if(p->at(i)>i)
			pair.append(1, '(');
		else
			pair.append(1, ')');		
	}
	
	return energy_of_struct(seq.c_str(),  pair.c_str());
	
}


/*get paired and unpaired sites */
void RNAWithPairingEvolModel::getSiteState(){

	vector<int> *tp = obj->getP();	//numerical representation of base pairing
	const vector<int> &p = *tp;		//NOTE: use reference here for easy grammar

	int plen = p.size();			

	/* clear vectors for storing various nucleotide states */
	pairedsite.clear();
	unpairedsite.clear();
	pairedsite.reserve(plen);
	unpairedsite.reserve(plen);

	/* paired or unpaired ? */
	for(int i = 0; i<plen; i++)
		p[i] >=0 ? pairedsite.push_back(i) : unpairedsite.push_back(i);		
	
}


/* compute R:
	R = R1*kappa/(1+kappa)
	where R1 = \sum{wr} is the total rate for substitution
	pool substitution rate to its distribution	
	also update Delta E */
double RNAWithPairingEvolModel::computeR(){

	vector<int> *tp = obj->getP();	//numerical representation of base pairing
	vector<int> *ts = obj->getS();  //site: 0, 1, 2,... size-1
	vector<double> *tw = obj->getW();  //site: 0, 1, 2,... size-1
	const vector<int> p = *tp;		//use reference for easy grammar, should not change them.
	const vector<int> s = *ts;
	const vector<double> w = *tw;
	
	int plen = p.size();
	int i=0;
	double ec=0.0;	//energy change caused by a mutation

	double R1 = 0.0;	//total transition rate for substitution
	
	/* Compute R1:total rate for substitution*/
	for(i=0; i<plen; i++){
		for(int j=0; j<=3; j++){
			if(j != s[i]){//change to a different nucleotide
				ec = substituteTest(i,j);
				if(ec<deltaE)	deltaE = ec;
				double r = 0.0;
				if(ec>=0.0 && ec<0.01 || ec<=0.0 && ec>-0.01)//neutral mutation
					r=mu;
				else{//non-neutral mutation		
					double s = exp(-1.0 * alpha * ec) - 1.0;
					double p  = (1.0 - exp(-2.0*s))/(1.0 - exp(-4.0 * N * s));
					r = 2.0 * N * mu * p;
				}	
				R1 += w[i] * r;
				
				subRates.push_back(w[i]*r);//pool weighted rates
			}
		}
	}
	
	/*check if Delta E is in insertion*/
	for(i=0; i<plen; i++)
		for(int j=0; j<=3; j++){
			ec = insertTest(i, j);
			if(ec<deltaE)	deltaE = ec;
		}
		
	/*check if Delta E is in deletion*/
	for(i=0; i<plen; i++){
		ec = deleteTest(i);
		if(ec<deltaE)	deltaE = ec;	
	}
		
	return R1*(kappa+1)/kappa;	//R = R1*(kappa+1)/kappa
}

/*estmate R by pool of substitution rates*/
double RNAWithPairingEvolModel::estimateR(){
	
	int plen = obj->getP()->size();
	int vs = subRates.size();	
	double R1 = 0.0;
		
	for(int i=0; i<3*plen; i++)//each site has three substitutions
		R1 += subRates.at(_randInt(0, vs-1));							
	
	return R1*(kappa+1)/kappa;	//R = R1*(kappa+1)/kappa
}


/* sample a site by weight */
int RNAWithPairingEvolModel::WeightedSampling() const {
	
	vector<double> *tw = obj->getW();	//weight vector: each site is tagged with a weight
	const vector<double> &w = *tw;
	int wlen = w.size();
	double tweight =0.0;
	int i=0;
	
	for(i=0; i<wlen; i++)
		tweight += w[i];
	double rd = tweight*gsl_rng_uniform(vstate); //gsl_rng_uniform() returns a random number uniformly distributed in [0, 1);
	tweight = 0.0;
	for(i=1; i<wlen-2; i++){
		tweight += w[i];
		if(rd<=tweight)				
			return i;			
	}
	return wlen-2;
	
}

/******************************************************************************************
	mutation on object obj with expected number of events; obj will be modified
	here, obj is RNAWithPairing
	--------------------------------------------------------------
		Mutation Schema
		1. make a time increment, if it is greater than the threshold, break.	
		2. draw an indel or a substitution by ratio kappa
		3. for the selected mutation type, draw a site by weight w
		4. accept/abort the mutation according to r/(r_max*c)
		5. go to next loop step until a mutation is accepted.
	 ---------------------------------------------------------------
******************************************************************************************/
void RNAWithPairingEvolModel::mutate_exevent(RNAWithPairing * object, double el, bool verboseflag, int nnodes){

	obj = object;
	this->getSiteState();//recompute site state
	
	/*compute folding free energy for wild type RNA*/
	E_0 = computeEnergy(obj->getP(), obj->getS());	

	if(NumMutatedRNA == 0){/*root RNA, create global E_opt*/
		double tmp = computeEnergy(obj->getP(), obj->getS());
		if(E_opt<0)
			E_opt = tmp - abs(tmp)*abs(E_opt);
		else
			E_opt = tmp + abs(tmp)*E_opt;
	}

	if(fixed_global_DeltaE)/*whether use fixed global DeltaE */
		deltaE = global_DeltaE; 

	/*compute r_max = 2Nup_max*/		
	double s_max = exp(-1.0 * alpha * deltaE) - 1.0;
  	double p_max = (1.0 - exp(-2.0*s_max))/(1.0 - exp(-4.0*N*s_max));
	double r_max  = 2.0 * N * mu * p_max;
					
	double T = 0.0;	//evolution time
	while(1){		
		/*compute total transition rate R, DeltaE is also updated*/

		double R = 0.0;	
		if(NumMutatedRNA<distr_pool_size){
			R = computeR();

			if(fixed_global_DeltaE)/*whether use fixed global DeltaE */
				deltaE = global_DeltaE; 

			if(NumMutatedRNA == 0){/*root RNA*/
				/*compute r_max = 2Nup_max*/
				s_max = exp(-1.0 * alpha * deltaE) - 1.0;
				p_max = (1.0 - exp(-2.0*s_max))/(1.0 - exp(-4.0*N*s_max));
				r_max  = 2.0 * N * mu * p_max;
			}
			NumMutatedRNA++;
		}else{	
			R = estimateR();
		}

	
		/*guard DeltaE against large negative values */
		if(deltaE<-10.0)	
			deltaE = -10.0;
		
		/* 1.make a time increment, waiting time is an exponential distribution with mean 1/R */		
		T += expdev(vstate, R);		
		if(T > el)	break;
		
		/* 2.draw an indel or a substitution by ratio kappa */
		/* 0: insertion, 1: deletion, 2:substitution */
		int muttype = 2;
		double arb = gsl_rng_uniform(vstate);//uniformly distributed in [0,1).		
		if(arb<kappa/(1.0+kappa)*0.5)	muttype = 0;
		else if(arb<kappa/(1.0+kappa))	muttype = 1;

		/* perform a mutation for the selection mutation type*/
		bool notaccepted = true;		
		while(notaccepted){
		
			int mutsite = 0;
			int mutnuc = 0;		//used in substitution			
			vector<int> mutnucs;//used in insertion
			int indelsize=0;
			double EC = 0.0;
		
			vector<int> *ts = obj->getS();	//numerical representation of sequence
			const vector<int> &s = *ts;
					
			/*3. draw a site by weight w and create nucleotide(s) to be mutated*/
			mutsite = WeightedSampling();
			if(muttype == 0){//insertion
				indelsize = getIndelLength();
				for(int i=0; i<indelsize; i++)
					mutnucs.push_back(_randInt(0,3));
				EC = multiinsertTest(mutsite, mutnucs);					
			}else if(muttype==1){//deletion
				indelsize = getIndelLength();
				if(mutsite+indelsize>=s.size())//prevent overflow
					indelsize = s.size() - mutsite - 1;
				EC = multideleteTest( mutsite, indelsize );				
			}else{//substitution
				mutnuc = (s[mutsite] + _randInt(1,3))%4;
				EC  = substituteTest( mutsite, mutnuc );
			}
			
			/*update deltaE if necessary*/
			if(EC<deltaE && EC>-10.0){//guard against -9999, the unexpected value
				if(!fixed_global_DeltaE){/*whether use fixed global DeltaE */
					deltaE = EC;
					s_max  = exp(-1.0 * alpha * deltaE) - 1.0;
					p_max  = (1.0 - exp(-2.0*s_max))/(1.0 - exp(-4.0*N*s_max));
					r_max  = 2.0 * N * mu * p_max;
				}
			}
			
			/*optimal energy control */
			if(E_0<E_opt)
				EC = -EC;
			
			/*4. accept/abort the mutation according to gamma = r/(r_max*c)*/			
			double r = 1.0; //fixation rate
			if(EC>=0.0 && EC<0.01 || EC<=0.0 && EC>-0.01){/* neutral change */
				r = mu;
			}else{/* non-neutral change */				
				double s = exp(-1.0 * alpha * EC)-1.0;
				double p = (1.0 - exp(-2.0*s))/(1.0 - exp(-4.0*N*s));
				r = 2.0 * N * mu * p;
			}
			double gamma = r/(r_max*gamma_maxweight);
		
			if(verbose)
				printf("\tgamma = %.6f global_DeltaE = %.3f  wildtype_energy = %.3f  current_DeltaE=%.3f  mutation type=%d indelsize=%d\n", gamma, deltaE, E_0, EC, muttype, indelsize);
			double ph = ranlxs_get_double(vstate);	//return a double uniformly distributed on [0,1)		
			if(ph<gamma){
				if(verbose)
				cout<<"\tcommit mutation :"<<muttype<<" at "<<mutsite<<endl;
				switch(muttype){
					case 0:
						multiinsertCommit(mutsite, mutnucs); 
						break;
					case 1: 
						multideleteCommit(mutsite, indelsize); 
						break;
					case 2:
						substituteCommit(mutsite, mutnuc);
						break;					
					default:
						cerr<<"Error: at at file "<<__FILE__<<" line "<<__LINE__
							<<"In RNAWithPairingEvolModel::mutate_exevent(): unknown event"<<endl;
						exit(1);
				}
				this->getSiteState();//recompute site state
	
				E_0 = computeEnergy(obj->getP(), obj->getS());//we have to recompute E_0 each time there is a mutation
				notaccepted = false;
			}		
		}
		
	}
}


//END_OF_FILE
	/*vector<int> *ts = obj->getS();	//numerical representation of sequence
		const vector<int> &s = *ts;
		vector<int> *tp = obj->getP();	//numerical representation of base pairing		
		const vector<int> &p = *tp;
		*/
		
