/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/


#include "Global.h"
#include "Utility.h"

#include <cstdlib>
#include <ctime>
#include <cmath>
#include <vector>
#include <iostream>

using namespace std;

extern ranlxs_state_t * vstate; //global variable defined in main.cpp

/* Random number generator from GNU Scientific Library  gsl/rng/ranlxs.c
   This is an implementation of M. Luescher's second generation
   version of the RANLUX generator.
   Thanks to Martin Luescher for providing information on this
   generator.

   According to GSL reference manual, this generator offers "the best mathematically-proven quality". This is good!
   ranlxs_get_double(..) return a double uniformly distributed on [0,1)

   Usage of the function   	
	
	ranlxs_state_t * vstate	//global definition

	int main(){
		
		vstate = new ranlxs_state_t;
		ranlxs_set_lux(vstate, static_cast<long>(time(0)), 397);	//initialize; second para is seeding
		...
		delete s;
	}
	
	extern ranlxs_state_t * vstate; //global variable defined in main.cpp
	void foo(){
		cout<<ranlxs_get_double(vstate)<<endl;
	}
 */

void increment_state (ranlxs_state_t * state)
{
  int k, kmax, m;
  double x, y1, y2, y3;

  float *xflt = state->xflt;
  double *xdbl = state->xdbl;
  double *ydbl = state->ydbl;
  double carry = state->carry;
  unsigned int ir = state->ir;
  unsigned int jr = state->jr;

  for (k = 0; ir > 0; ++k)
    {
      y1 = xdbl[jr] - xdbl[ir];
      y2 = y1 - carry;
      if (y2 < 0)
        {
          carry = one_bit;
          y2 += 1;
        }
      else
        {
          carry = 0;
        }
      xdbl[ir] = y2;
      ir = next[ir];
      jr = next[jr];
    }

  kmax = state->pr - 12;

  for (; k <= kmax; k += 12)
    {
      y1 = xdbl[7] - xdbl[0];
      y1 -= carry;

      RANLUX_STEP (y2, y1, 8, 1, 0);
      RANLUX_STEP (y3, y2, 9, 2, 1);
      RANLUX_STEP (y1, y3, 10, 3, 2);
      RANLUX_STEP (y2, y1, 11, 4, 3);
      RANLUX_STEP (y3, y2, 0, 5, 4);
      RANLUX_STEP (y1, y3, 1, 6, 5);
      RANLUX_STEP (y2, y1, 2, 7, 6);
      RANLUX_STEP (y3, y2, 3, 8, 7);
      RANLUX_STEP (y1, y3, 4, 9, 8);
      RANLUX_STEP (y2, y1, 5, 10, 9);
      RANLUX_STEP (y3, y2, 6, 11, 10);

      if (y3 < 0)
        {
          carry = one_bit;
          y3 += 1;
        }
      else
        {
          carry = 0;
        }
      xdbl[11] = y3;
    }

  kmax = state->pr;

  for (; k < kmax; ++k)
    {
      y1 = xdbl[jr] - xdbl[ir];
      y2 = y1 - carry;
      if (y2 < 0)
        {
          carry = one_bit;
          y2 += 1;
        }
      else
        {
          carry = 0;
        }
      xdbl[ir] = y2;
      ydbl[ir] = y2 + shift;
      ir = next[ir];
      jr = next[jr];
    }

  ydbl[ir] = xdbl[ir] + shift;

  for (k = next[ir]; k > 0;)
    {
      ydbl[k] = xdbl[k] + shift;
      k = next[k];
    }

  for (k = 0, m = 0; k < 12; ++k)
    {
      x = xdbl[k];
      y2 = ydbl[k] - shift;
      if (y2 > x)
        y2 -= sone_bit;
      y1 = (x - y2) * sbase;

      xflt[m++] = (float) y1;
      xflt[m++] = (float) y2;
    }

  state->ir = ir;
  state->is = 2 * ir;
  state->is_old = 2 * ir;
  state->jr = jr;
  state->carry = carry;
}


double ranlxs_get_double (void *vstate)
{
  ranlxs_state_t *state = (ranlxs_state_t *) vstate;

  const unsigned int is = snext[state->is];

  state->is = is;

  if (is == state->is_old)
    increment_state (state);

  return state->xflt[state->is];
}

/*unsigned long int ranlxs_get (void *vstate)
{
  return ranlxs_get_double (vstate) * 16777216.0;       // 2^24 
}*/

void ranlxs_set_lux (void *vstate, unsigned long int s, unsigned int luxury)
{
  ranlxs_state_t *state = (ranlxs_state_t *) vstate;
  int ibit, jbit, i, k, m, xbit[31];
  double x, y;
  long int seed;

  if (s == 0)
    s = 1;                      /* default seed is 1 */

  seed = s;
  i = seed & 0xFFFFFFFFUL;
  for (k = 0; k < 31; ++k){
      xbit[k] = i % 2;
      i /= 2;
  }

  ibit = 0;
  jbit = 18;

  for (k = 0; k < 12; ++k)
    {
      x = 0;

      for (m = 1; m <= 48; ++m)
        {
          y = (double) xbit[ibit];
          x += x + y;
          xbit[ibit] = (xbit[ibit] + xbit[jbit]) % 2;
          ibit = (ibit + 1) % 31;
          jbit = (jbit + 1) % 31;
        }
      state->xdbl[k] = one_bit * x;
    }

  state->carry = 0;
  state->ir = 0;
  state->jr = 7;
  state->is = 23;
  state->is_old = 0;
  state->pr = luxury;
}





/*	Returns an exponentially distributed, positive, random deviate of mean 1/lamda, 
	using ranlxs_get_double(vstate) as the source of uniform deviates. 
*/
double expdev(ranlxs_state_t * vstate, double lamda){ //NOTE: caller needs to make sure lamda > 0
	double x;
	do{
		x=ranlxs_get_double(vstate);
	}while (x == 0.0);
	return -log(x)/lamda;	//x is uniformly distributed in (0.0, 1]
}




/* gamma distribution from GNU Scientific Library  gsl/randist/gamma.c 
   usage: gsl_ran_gamma(ranlxs_state_t * r, const double a, const double b);
   for shape of pdf, see:
   http://en.wikipedia.org/wiki/Gamma_distribution
   where 'k' is our 'a', and 'theta' is our 'b'
*/
double gsl_rng_uniform(ranlxs_state_t *vstate){//returns a double precision floating point number uniformly distributed in the range [0,1).
	return ranlxs_get_double (vstate);
}

double gsl_rng_uniform_pos( ranlxs_state_t *vstate){//returns a double precision floating point number uniformly distributed in the range (0,1).
	double x = ranlxs_get_double(vstate);
	while(x<0.00000000000000001){
		x = ranlxs_get_double(vstate);
	}
	return  x;
}

/* The Gamma distribution of order a>0 is defined by:

   p(x) dx = {1 / \Gamma(a) b^a } x^{a-1} e^{-x/b} dx

   for x>0.  If X and Y are independent gamma-distributed random
   variables of order a1 and a2 with the same scale parameter b, then
   X+Y has gamma distribution of order a1+a2.

   The algorithms below are from Knuth, vol 2, 2nd ed, p. 129. */

double gsl_ran_gamma (ranlxs_state_t * r, const double a, const double b)
{
  /* assume a > 0 */
  unsigned int na = (unsigned int)(floor (a));

  if (a == na)
    {
      return b * gsl_ran_gamma_int (r, na);
    }
  else if (na == 0)
    {
      return b * gamma_frac (r, a);
    }
  else
    {
      return b * (gsl_ran_gamma_int (r, na) + gamma_frac (r, a - na)) ;
    }
}

double gsl_ran_gamma_int (ranlxs_state_t * r, unsigned int a)
{
  if (a < 12)
    {
      unsigned int i;
      double prod = 1;

      for (i = 0; i < a; i++)
        {
          prod *= gsl_rng_uniform_pos (r);
        }

      /* Note: for 12 iterations we are safe against underflow, since
         the smallest positive random number is O(2^-32). This means
         the smallest possible product is 2^(-12*32) = 10^-116 which
         is within the range of double precision. */

      return -log (prod);
    }
  else
    {
      return gamma_large (r, (double) a);
    }
}

double gamma_large (ranlxs_state_t * r, double a)
{
  /* Works only if a > 1, and is most efficient if a is large

     This algorithm, reported in Knuth, is attributed to Ahrens.  A
     faster one, we are told, can be found in: J. H. Ahrens and
     U. Dieter, Computing 12 (1974) 223-246.  */

  double sqa, x, y, v;
  sqa = sqrt (2 * a - 1);
  do
    {
      do
        {
          y = tan (MM_PI * gsl_rng_uniform (r));
          x = sqa * y + a - 1;
        }
      while (x <= 0);
      v = gsl_rng_uniform (r);
    }
  while (v > (1 + y * y) * exp ((a - 1) * log (x / (a - 1)) - sqa * y));

  return x;
}

double gamma_frac (ranlxs_state_t * r, double a)
{
  /* This is exercise 16 from Knuth; see page 135, and the solution is
     on page 551.  */

  double p, q, x, u, v;
  p = MM_E / (a + MM_E);
  do
    {
      u = gsl_rng_uniform (r);
      v = gsl_rng_uniform_pos (r);

      if (u < p)
        {
          x = exp ((1 / a) * log (v));
          q = exp (-x);
        }
      else
        {
          x = 1 - log (v);
          q = exp ((a - 1) * log (x));
        }
    }
  while (gsl_rng_uniform (r) >= q);

  return x;
}









//cumulative sum of a vector
vector<double> cumsum(vector<double> x){

	double s = 0.0;
	int xsize = x.size();
	vector<double> y(xsize);	//create a vector of the same length as x

	for(int i = 0; i < xsize; i++){
		s += x[i];
		y[i] = s;	
	}
	return y;
}


//cumulative sum of a vector, will normalize the cumulative sum vector so it becomes a distribution
vector<double> cumprob(vector<double> x){

	int xsize = x.size();
	vector<double> cp(xsize);
	vector<double> y = cumsum(x);

	double n = y[xsize-1];
	for(int i = 0; i<xsize; i++)
		cp[i] = y[i]/n;
	
	return cp;	
}

//sample k observations from vector x with probability prob, is OK even if prob is not normalized (i.e. sum(prob)=1)
//WARNING: THE ORIGINAL PYTHON PROGRAM HAS A BUG IN THIS FUNCTION!
vector<double> sample(vector<double> x, vector<double> prob, int k){

	vector<double> dst = cumprob(prob);
	vector<double> samp(k);


	for(int i=0; i<k; i++){
		//double r= ran4(&ran4seed);	//generate a random number in [0.0, 1.0)
		double r = ranlxs_get_double(vstate); //generate a random number in [0.0, 1.0)
		int dsize = dst.size();
		int j;
		for(j=0; j < dsize; j++)
			if(r < dst[j])
				break;
		samp[i]= x[j];
	}
	return samp;
}

//rejection method with probability prob, is OK even if prob is not normalized. (i.e. sum(prob)=1)
int  rejectprob(vector<double> prob){
	
	vector<double> dst = cumprob(prob);
	int len = dst.size();
	//double r= ran4(&ran4seed); //generate a random number with uniform distribution in [0.0, 1.0)
	double r = ranlxs_get_double(vstate); //generate a random number in [0.0, 1.0)

	for(int i = 0; i< len; i++)
		if(r <= dst[i])
			return i;
	return 0;
}
