/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/

#ifndef UTILITY_H
#define UTILITY_H

#include <vector>
#include <iostream>
#include <sstream>

using namespace std;

#define MM_PI       3.14159265358979323846264338328      /* pi */
#define MM_E        2.71828182845904523536028747135      /* e */

enum ErrorType {FATAL=1,  WARN=2, IGNORE=3};	

/* random number generator */
double ranlxs_get_double (void *vstate);
void ranlxs_set_lux (void *state, unsigned long int s, unsigned int luxury);

static const int next[12] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0};
static const int snext[24] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
                              14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 0};

static const double sbase = 16777216.0;         /* 2^24 */
static const double sone_bit = 1.0 / 16777216.0;        /* 1/2^24 */
static const double one_bit = 1.0 / 281474976710656.0;  /* 1/2^48 */

static const double shift = 268435456.0;        /* 2^28 */

#define RANLUX_STEP(x1,x2,i1,i2,i3)      \
          x1=xdbl[i1] - xdbl[i2];        \
          if (x2 < 0)                    \
          {                              \
            x1-=one_bit;                 \
            x2+=1;                       \
          }                              \
          xdbl[i3]=x2

typedef struct
  {
    double xdbl[12], ydbl[12];  /* doubles first so they are 8-byte aligned */
    double carry;
    float xflt[24];
    unsigned int ir;
    unsigned int jr;
    unsigned int is;
    unsigned int is_old;
    unsigned int pr;
  }
ranlxs_state_t;

void increment_state (ranlxs_state_t * state);

/* gamma random variable */
double gamma_large (ranlxs_state_t * r, const double a);
double gamma_frac (ranlxs_state_t * r, const double a);
double gsl_ran_gamma_int (ranlxs_state_t * r, unsigned int a);
double gsl_ran_gamma (ranlxs_state_t * r, double a, double b);
double gsl_rng_uniform(ranlxs_state_t *vstate);//returns a double precision floating point number uniformly distributed in the range [0,1).
double gsl_rng_uniform_pos( ranlxs_state_t *vstate);//returns a double precision floating point number uniformly distributed in the range (0,1).


//returns an exponentially distributed, positive, random deviate of mean 1/lamda  taken from Numerical Recipes in C */
//double expdev(long *idum, double lamda);
double expdev(ranlxs_state_t * vstate, double lamda); //NOTE: caller needs to make sure lamda > 0

//cumulative sum of a vector
vector<double> cumsum(vector<double> x);

//cumulative sum of a vector, will normalize the cumulative sum vector so it becomes a distribution
vector<double> cumprob(vector<double> x);

//sample k observations from vector x with probability prob, is OK even if prob is not normalized (i.e. sum(prob)=1)
vector<double> sample(vector<double> x, vector<double> prob, int k=1);

//rejection method with probability prob, is OK even if prob is not normalized. (i.e. sum(prob)=1)
int rejectprob(vector<double> prob);



/* remove leading and trailing whitespace */
static inline void removeLeadTrailWS(string & tmp){	
	while (tmp.find(" ") == 0) 
		tmp.erase(0, tmp.find(" ") + 1);

	while((tmp.rfind(" ") == (tmp.length() - 1)) && (tmp.length() > 0))
		tmp.erase(tmp.rfind(" "), tmp.length());
}

/* repalce char s by t in a string */
static inline void strReplace(string & tmp, const char s, const char t){
	int len = tmp.size();
	for(int i = 0; i<len; i++)
		if(tmp[i] == s)
			tmp.replace(i, 1, 1, t); 
}

/* convert string to double */
static inline double strToDouble(const string & s){ 
	return atof( s.c_str() );
}

/* check if an element is in a vector */
template<class T>
static inline bool _isInVector(vector<T> &v, T x){	
	int vlen=v.size();
	for(int i = 0; i<vlen; i++)
		if(v[i]==x)
			return true;
	return false;
}


/*	insert x as the (i+1)'th element of v, i.e. x has index i*/
template<class T>
static inline void _insertToVector(vector<T> &v, int i, T x){
	typename  vector<T>::iterator iter = v.begin();
	if(i<0)	return;
	if(i>=v.size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		v.push_back(x);
	}else{
		for(int k=0; k<i; k++)
			iter++;
		v.insert(iter, x);
	}
}


/* remove i-indexed (the i+1th) element from a vector */
template<class T>
static inline void _removeFromVector(vector<T> &v, unsigned int i){
	typename vector<T>::iterator iter = v.begin();
	if(i<0 || v.size()==0)	
		return;
	if(i>=v.size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		return;			//i-indexed element does not exist
	}else if(i == v.size()-1){
		v.pop_back();
	}else{
		for(unsigned int k=0; k<i; k++)
			iter++;
		//now iter = i;
		v.erase(iter);
	}
}

/* remove elements with value x from a vector */
template<class T>
static inline void _removeFromVectorX(vector<T> &v, T x){
	
	/*for(int i =0; i<v.size(); i++)
		if(v[i]==x){
			_removeFromVector(v, i);
			return;
		}*/

	for(int i=v.size()-1; i>=0; i--)
		if(v[i]==x)
			_removeFromVector(v, i);
}

template<class T>
static inline void _swap(T *i, T *j){
	T k;
	k = *i;
	*i = *j;
	*j = k;
}

template<class T>
static inline T _min(T x, T y) { return (x <y) ? x : y; }

static inline double _abs(double s){
	if(s>0.0)
		return s;
	else
		return -1.0*s;
}

template<typename T> 
static inline string NumToStr(T x){
	std::ostringstream o;
	if (!(o << x)){
		/*string errmsg("In ");
		errmsg.append(__FILE__);
		errmsg.append(" line ");
		errmsg.append(__LINE__);
		errmsg.append(" : Unable to convert ");
		errmsg.append(x);
		errmsg.append(" to string");*/
		//errProcess(FATAL,errmsg);
	}	
	return o.str();
}



#endif
//END_OF_FILE
