/* Copyright (c) 2008, Sheng Guo, Li-San Wang and Junhyong Kim, University of
  Pennsylvania.  All Rights Reserved.
 
  You may not use this file except in compliance with the License
  located in the top directory of this distribution.  You may obtain
  a copy of the License at
 
      http://kim.bio.upenn.edu/software/index.shtml
 
  Unless required by applicable law or agreed to in writing, this
  software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
  CONDITIONS OF ANY KIND, either express or implied.  See the License
  for the specific language governing permissions and limitations
  under the License. 
*/

/*******************************************************************************************
Create true alignement from rnasim output files
*********************************************************************************************/

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <vector>
#include <cassert>
#include <ctime>
#include <iomanip>

using namespace std;

/* remove leading and trailing whitespace */
static inline void removeLeadTrailWS(string & tmp){

	if(tmp.size()==0) return;
	tmp.erase( 0, tmp.find_first_not_of( " \t\n" ) );
	tmp.erase( tmp.find_last_not_of( " \t\n" ) + 1);
}



/* check if a string contains character c*/
static inline bool hasCharacter(string & tmp, char c){
	unsigned int loc = tmp.find(c, 0 );
	if( loc != string::npos )
		return true;
	else
		return false;		
}

/* convert a string to an integer */
static inline int StrToInt(const string & s){
	return atoi( s.c_str() );	
}

static inline int max(int x, int y) { return (x>y) ? x : y; }

static inline void StringTokenizer(const string& str, vector<string> * tokens, const string& delimiters = " "){
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);
    while (string::npos != pos || string::npos != lastPos)    {
        // Found a token, add it to the vector.
        tokens->push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}

static inline void StringTokenizer2(const string& str, vector<int> * tokens, const string& delimiters = " "){
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);
    while (string::npos != pos || string::npos != lastPos)    {
        // Found a token, add it to the vector.
        tokens->push_back(StrToInt(str.substr(lastPos, pos - lastPos)));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}


/* check if the vector contains only integers no greater than i */
static inline bool checkCode(const vector<int> &tokens, int i){
	int tsize = tokens.size();
	if(tsize<1)
		return false;	
	for(int j=0; j<tsize; j++)
		if(tokens[j]>i)
			return false;
	return true;
}

/* check if the vector contains only integers i and i-1 */
static inline bool checkCode2(const vector<int> &tokens, int i){
	int tsize = tokens.size();
	if(tsize<1)
		return false;
	for(int j=0; j<tsize; j++)
		if(!(tokens[j]==i || tokens[j]==i-1))
			return false;
	return true;
}

/* determine the maximal code in a parenthesis code */
static inline int maxCode(const string &tmp){	
	int x=0;
	vector<string> * t = new vector<string>;
	t->reserve(30);
	StringTokenizer(tmp, t, ",()");
	for(unsigned int i=0; i<t->size(); i++){
		int y = StrToInt(t->at(i));
		if(y>x)
			x=y;
	}
	delete t;
	return x;
}

/* determine the minimal code in a parenthesis code */
static inline int minCode(const string &tmp){	
	int x=9999999;
	vector<string> * t = new vector<string>;
	t->reserve(30);
	StringTokenizer(tmp, t, ",()");
	for(unsigned int i=0; i<t->size(); i++){
		int y = StrToInt(t->at(i));
		if(y<x)
			x=y;
	}
	delete t;
	return x;
}



/* determine number of left most parenthesis of an identifier
	e.g 3 for (((28,29),29),29)  */	
static inline int numOfLP(string & tmp){
	int tmpsize = tmp.size();
	if(tmpsize==0)
		return 0;
	int i=0;
	while(i<tmpsize && tmp[i]=='(')
		i++;
	return i;
}

/* determine number of right most parenthesis of an identifier
	e.g 2 for (1, (1,2))  */	
static inline int numOfRP(string & tmp){
	int tmpsize = tmp.size();
	if(tmpsize==0)
		return 0;
	int i=tmp.size()-1;
	int x = 0;
	while(i>=0 && tmp[i]==')'){
		i--;
		x++;
	}
	return x;
}

/* check if an element is in a vector */
template<typename T>
static inline bool isInVector(vector<T> &v, T x){	

	for(unsigned int i = 0; i<v.size(); i++)
		if(v[i]==x)
			return true;
	return false;
}

/* remove i-indexed element from a vector */
/*template<class T>
static inline void removeFromVector(vector<T> &v, int i){
	if(i<0)	return;
	typename vector<T>::iterator iter = v.begin();	
	if(i>=v.size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		return;
	}else if(i == v.size()-1){
		v.pop_back();
	}else{
		for(int k=0; k<i; k++)
			iter++;
		//now iter = i;
		v.erase(iter);
	}
}*/

/*	insert x as the (i+1)'th element of v, i.e. x has index i*/
/*template<class T>
static inline void insertToVector(vector<T> * v, int i, T x){
	if(i<0)	return;
	typename vector<T>::iterator iter = v->begin();	
	if(i>=v->size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		iter = v->end();
		v->push_back(x);
	}else{
		for(int k=0; k<i; k++)
			iter++;
		v->insert(iter, x);
	}
}*/

static inline void removeFromVector(vector<int> &v, int i){
	if(i<0)	return;
	vector<int>::iterator iter = v.begin();	
	if(i>=v.size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		return;
	}else if(i == v.size()-1){
		v.pop_back();
	}else{
		for(int k=0; k<i; k++)
			iter++;
		//now iter = i;
		v.erase(iter);
	}
}


static inline void insertToVector(vector<string> * v, int i, string x){
	if(i<0)	return;
	vector<string>::iterator iter = v->begin();	
	if(i>=v->size()){	//must guarantee not breaking boundary of the vector, or get 'segmentation fault'
		iter = v->end();
		v->push_back(x);
	}else{
		for(int k=0; k<i; k++)
			iter++;
		v->insert(iter, x);
	}
}


static inline bool isSimpleCode(string &tmp){
	int tmpsize=tmp.size();
	if(tmpsize<2)
		return true;
	if(tmp[tmpsize-1]==')' && tmp[tmpsize-2]!=')' )
		return true;
	return false;
}


/* split complex code into two composite parts
e.g		 (1369,(1369,1370)) --> 1369 and (1369,1370)
*/
static inline void splitComplexCode(const string & orig, string * head, string * tail){

	//(310,311),((310,311),311)
	string tmp = orig.substr(1, orig.size()-2);	//remove leading and tailing parenthesis	
	int si=0; //split index
	int x = 0;
	int flag = 1;
	while(flag!=0 || x!=0 ){		
		if(tmp[si]=='('){
			x++;
			flag=1;
		}else if(tmp[si]==')'){
			x--;
			flag=1;
		}else if(tmp[si]==',')
			flag=0;
		else
			flag=1;
		si++;
	}
	
	*head = tmp.substr(0, si-1);
	*tail = tmp.substr(si);
}

/* bubble-sort code by length/size */
static inline void sortBySize(vector<string> * code){
	int n = code->size();
	for(int i=0; i<n-1; i++){
		for(int j=0; j<n-1-i; j++)
			if(code->at(j+1).length() < code->at(j).length()){
				string tmp = code->at(j);
				code->at(j) = code->at(j+1);
				code->at(j+1) = tmp;
			}
	}
}

/* sort parenthesis-encoded uniqcode */
/* three examples:
	1369 (1369,(1369,1370)) (1369,1370) ((1369,1370),1370) 1370 
	
	310 (310,311) ((310,311),((310,311),311)) ((310,311),311) (((310,311),311),311) ((((310,311),311),311),311) 
	(((((310,311),311),311),311),311) ((((((310,311),311),311),311),311),311) (((((((310,311),311),311),311),311),311),311) 
	((((((((310,311),311),311),311),311),311),311),311) 311 312
	
	1016 (1016,1017) ((1016,1017),1017) (((1016,1017),1017),1017) ((((1016,1017),1017),1017),1017) 
	(((((1016,1017),1017),1017),1017),(((((1016,1017),1017),1017),1017),1017)) (((((1016,1017),1017),1017),1017),1017) 
	((((((1016,1017),1017),1017),1017),1017),1017) (((((((1016,1017),1017),1017),1017),1017),1017),1017) 1017 */
static inline void sortUniqcode(vector<string> * code){
	
	unsigned int csize = code->size();
	if(csize==0)
		return; 

	sortBySize(code);
	
	vector<int> complexCodeInd;
	vector<string> * newcode = new vector<string>;	
	complexCodeInd.reserve(code->size());
	newcode->reserve(code->size());	
			
	/* sort simple code first */
	for(unsigned int i=0; i<csize; i++){
		string tmp = code->at(i);
		//if(tmp[tmp.size()-1]==')' && tmp[tmp.size()-2]!=')' ){//simple code		
		if(tmp[tmp.size()-2]!=')' ){//simple code		
			int tx = newcode->size()-1;
			while(tx>=0 && newcode->at(tx).size()==tmp.size() && minCode(newcode->at(tx)) > minCode(tmp))
				tx--;			
			insertToVector(newcode, tx+1, tmp);
		}
		else
			complexCodeInd.push_back(i);
	}	
    
	
	/* insert complex code into sorted simple code */
	int cnrp=2;	//current number of right parenthesis
	while(complexCodeInd.size()>0){
		vector<int> curCodeInd;
		vector<int> removedInd;
		curCodeInd.reserve(complexCodeInd.size());
		removedInd.reserve(complexCodeInd.size());
		for(unsigned int i=0; i<complexCodeInd.size(); i++){
			int nrp = numOfRP(code->at(complexCodeInd[i]));
			//cout<<nrp<<endl;
			//cout<<code->at(complexCodeInd[i])<<endl;
			if(nrp==cnrp){
				curCodeInd.push_back(complexCodeInd[i]);
				removedInd.push_back(i);				
			}		
		}
		
		//cout<<removedInd.size()<<endl;
		for(int j=removedInd.size()-1; j>=0; --j){
			//cout<<"removing "<<removedInd[j]<<endl;
			removeFromVector(complexCodeInd, removedInd[j]);				
		}

		for(unsigned int q=0; q<curCodeInd.size(); q++){
			/* split code into to two composition parts */
			string orig = code->at(curCodeInd[q]);
			string head, tail;			
			splitComplexCode(orig, &head, &tail);
			bool isInserted = false;
			/* find the postion for tmp in sorted code */
			if(numOfLP(orig)==1)
				insertToVector(newcode, 0, orig);
			else{
				unsigned int k=0;
				/*search for exact mathces of head and tail*/
				while(k<newcode->size()){					
					if(newcode->at(k)==head){
						insertToVector(newcode, k+1, orig);				
						isInserted = true;
						break;
					}else if(newcode->at(k)==tail){					
						if(k==0)
							insertToVector(newcode, 0, orig);				
						else
							insertToVector(newcode, k, orig);				
						isInserted = true;
						break;	
					}else
						k++;					
				}	
				/* no exact match for head or tail */
				if(isInserted ==false){
					while(k<newcode->size() && newcode->at(k).size()<=head.size())
						k++;						
					insertToVector(newcode, k, orig);						
				}
			}			
		}
		cnrp++;
	}
	
	*code = *newcode;
	delete newcode;
}

/* sort code containing '-1' */
static inline void sortUniqcodeMOne(vector<string> * code){
	
	unsigned int csize = code->size();
	if(csize==0)
		return; 

	sortBySize(code);
	vector<int> complexCodeInd;
	vector<string> * newcode = new vector<string>;	
	complexCodeInd.reserve(code->size());
	newcode->reserve(code->size());	
	unsigned int i = 0;

	/* extract -1 first */
	for(i=0; i<csize; i++)		
		if(code->at(i) == "-1")
			newcode->push_back("-1");    
	if(newcode->size() ==  code->size())
		return;

	/* sort simple code first */
	for(i=0; i<csize; i++){
		string tmp = code->at(i);
		//if(tmp[tmp.size()-1]==')' && tmp[tmp.size()-2]!=')' ){//simple code		
		if(tmp[tmp.size()-2]!=')' ){//simple code		
			int tx = newcode->size()-1;
			while(tx>=0 && newcode->at(tx).size()==tmp.size() && minCode(newcode->at(tx)) > minCode(tmp))
				tx--;			
			insertToVector(newcode, tx+1, tmp);			
		}else
			complexCodeInd.push_back(i);
	}	
    
	/* insert complex code into sorted simple code */
	int cnrp=2;	//current number of right parenthesis
	while(complexCodeInd.size()>0){
		vector<int> curCodeInd;
		vector<int> removedInd;
		curCodeInd.reserve(complexCodeInd.size());
		removedInd.reserve(complexCodeInd.size());
		for(unsigned int i=0; i<complexCodeInd.size(); i++){
			int nrp = numOfRP(code->at(complexCodeInd[i]));
			//cout<<nrp<<endl;
			//cout<<code->at(complexCodeInd[i])<<endl;
			if(nrp==cnrp){
				curCodeInd.push_back(complexCodeInd[i]);
				removedInd.push_back(i);				
			}		
		}
		
		//cout<<removedInd.size()<<endl;
		for(int j=removedInd.size()-1; j>=0; --j){
			//cout<<"removing "<<removedInd[j]<<endl;
			removeFromVector(complexCodeInd, removedInd[j]);				
		}

		for(unsigned int q=0; q<curCodeInd.size(); q++){
			/* split code into to two composition parts */
			string orig = code->at(curCodeInd[q]);
			string head, tail;			
			splitComplexCode(orig, &head, &tail);
			bool isInserted = false;
			/* find the postion for tmp in sorted code */
			if(numOfLP(orig)==1)
				insertToVector(newcode, 0, orig);
			else{
				unsigned int k=0;
				/*search for exact mathces of head and tail*/
				while(k<newcode->size()){					
					if(newcode->at(k)==head){
						insertToVector(newcode, k+1, orig);				
						isInserted = true;
						break;
					}else if(newcode->at(k)==tail){					
						if(k==0)
							insertToVector(newcode, 0, orig);				
						else
							insertToVector(newcode, k, orig);				
						isInserted = true;
						break;	
					}else
						k++;					
				}	
				/* no exact match for head or tail */
				if(isInserted ==false){
					while(k<newcode->size() && newcode->at(k).size()<=head.size())
						k++;						
					insertToVector(newcode, k, orig);						
				}
			}			
		}
		cnrp++;
	}
	
	*code = *newcode;
	delete newcode;
}

/*return the one + index of the last element with interger indentifier less than ub */
static inline int lastIndex(const vector<string> * v, int startindex, int ub){

	unsigned int i=startindex;
	while(i<v->size()){
		vector<int> tokens;
		tokens.reserve(10);
		StringTokenizer2(v->at(i), &tokens, ",()");
		if(checkCode(tokens, ub-1)) // check if the vector contains only integers no greater than ub
			i++;
		else
			break;
	}	
	return i;
}


/* copy source[si](included) to source[ei](not included) to target */
static inline void partialCopy(vector<string> * target, vector<string> * source, int si, int ei){
	for(int i=si; i<ei; i++)
		target->push_back(source->at(i));
}


/* main program */
int main(int argc, char* argv[]){		

	
	//clock_t tt1=clock();
	cerr<<"truealign version 1.0"<<endl;
	cerr<<"--Creating homologous alignment for rnasim output files"<<endl<<endl;

	if(argc!=5){ 
		cerr<<"Usage: truealign <rnasim.out.nuc>  <rnasim.out.pos>  <rnasim.out.sid>  <alignment>"<<endl; 		
		cerr<<"\t<rnasim.out.nuc>  <rnasim.out.pos>  <rnasim.out.sid> are the output files generated by rnasim"<<endl;
		cerr<<"\t<alignment> is the file name given to the alignment"<<endl;	
		exit(1);
	}    	
	
	/* define variables */	
	char * seqfile   = argv[1];
	char * codefile  = argv[2];
	char * labelfile = argv[3];
	char * outfile   = argv[4];
	int m = 0;		//number of sequence
	int n = 0;		//maximal code value, for ecoli_ssu_rRNA n=1542
	vector< vector<string> *> uniqcode; //unique parenthesis code before each integer code
	vector<string> label;
	int i;
	
	remove(outfile);
	ofstream out(outfile, ios::app);
	if(!out){ cerr<<"file "<<outfile<<" can not be opened!"<<endl; exit(1);}

	/* get maximal code value */
	string codestr;		
	ifstream inCode(codefile, ios::in);
	if(!inCode){cerr<<"file "<<codefile<<" can not be opened!"<<endl; exit(1);}
	while(getline(inCode,codestr)){
		removeLeadTrailWS(codestr);
		if(codestr.size()>0){
			int k = codestr.size()-1;
			if(codestr[k]!= ')'){
				while(codestr[k]!=' ')
					k--;
				int x = StrToInt(codestr.substr(k));
				if(n<x)	n=x;
			}
			m++;
		}
	} 


	inCode.close();
	//cout<<"maximal code "<<n<<endl;
	cerr<<"Num of sequence="<<m<<endl;
	
	/* read in label: one label sequence per line */	
	label.reserve(m);
	string labelstr;		
	ifstream inLabel(labelfile, ios::in);
	if(!inLabel){cerr<<"file "<<labelfile<<" can not be opened!"<<endl;	exit(1);}
	while(getline(inLabel,labelstr)){
		removeLeadTrailWS(labelstr);
		if(labelstr.size()>0)
			label.push_back(labelstr);
	} 
	inLabel.close();
	//cout<<"finish reading in label sequences!"<<endl; 
	
	/* initialize uniqcode */
	for(i=0; i<=n; i++){
		vector<string> * t = new vector<string>;
		t->reserve(5);
		//assert(t);
		uniqcode.push_back(t);
	}
	
	/* get uniqcode */
	inCode.clear();
	inCode.open(codefile, ios::in);	
	if(!inCode){cerr<<"file "<<codefile<<" can not be opened!"<<endl;	exit(1);}	
	while(getline(inCode,codestr)){
		removeLeadTrailWS(codestr);
		if(codestr.size()>0){
			vector<string> * t = new vector<string>;
			t->reserve(2000);			
			StringTokenizer(codestr, t);
			unsigned int k=0;		
			int i=0;			
			/* process any code having -1 */
			while(hasCharacter(t->at(i),'-')){
				if(!isInVector(*uniqcode.at(0), t->at(i)))
					uniqcode.at(0)->push_back(t->at(i));
				k++;
				i++;				
			}	
			if(k>uniqcode.at(0)->size()){
				unsigned int ik = k-uniqcode.at(0)->size();
				while(ik>0){
					uniqcode.at(0)->push_back("-1");
					ik--;
				}
			}
			/* exam any code with parenthesis after 0 */
			int tsize = t->size();
			while(i<tsize){				
				if(hasCharacter(t->at(i), ')')){
					int j = maxCode(t->at(i));
					if(!isInVector(*uniqcode.at(j), t->at(i)))
						uniqcode.at(j)->push_back(t->at(i));
				}
				i++;
			}
			delete t;
		}
	} 
	inCode.close();
	
	/*sort uniqcode */
	//cout<<"start sorting "<<endl;
	sortUniqcodeMOne(uniqcode.at(0));/* deal with the special case: code containing '-1' */	
	/*for(unsigned int j=0; j<uniqcode.at(0)->size(); j++)
		cout<<uniqcode.at(0)->at(j)<<endl;*/
	for(i=1; i<=n; i++){/* deal with general case */		
		sortUniqcode(uniqcode.at(i));
		/*if(uniqcode.at(i)->size()!=0){
			for(int j=0; j<uniqcode.at(i)->size(); j++)
				cout<<uniqcode.at(i)->at(j)<<endl;
			cout<<endl;
		}*/
	}

	/*for(unsigned int i=0; i<uniqcode.size(); i++){
		vector<string> * t = uniqcode.at(i);
		cout<<i<<"\t";
		for(unsigned int j=0; j<t->size(); j++){
			cout<<t->at(j)<<"\t";
		}
		cout<<endl;
	}*/

	/* align sequence based on uniqcode */
	/* read in sequences: one sequence per line */
	cerr<<"Aligning in progress: ";
	string seqstr;		
	ifstream inSeq(seqfile, ios::in);
	if(!inSeq){cerr<<"file "<<seqfile<<" can not be opened!"<<endl;	exit(1);}
	inCode.clear();
	inCode.open(codefile, ios::in);	
	if(!inCode){cerr<<"file "<<codefile<<" can not be opened!"<<endl; exit(1);}	
	
	int lc=0; //line counter 
	while(getline(inCode,codestr) && getline(inSeq,seqstr)){

		if(lc == static_cast<int>(m *  0.1) ){
			cerr<<".....10%";
		}else if(lc == static_cast<int>(m *  0.2) ){
			cerr<<".....20%";
		}else if(lc == static_cast<int>(m *  0.3) ){
			cerr<<".....30%";
		}else if(lc == static_cast<int>(m *  0.4) ){
			cerr<<".....40%";
		}else if(lc == static_cast<int>(m *  0.5) ){
			cerr<<".....50%";
		}else if(lc == static_cast<int>(m *  0.6) ){
			cerr<<".....60%";
		}else if(lc == static_cast<int>(m *  0.7) ){
			cerr<<".....70%";
		}else if(lc == static_cast<int>(m *  0.8) ){
			cerr<<".....80%";
		}else if(lc == static_cast<int>(m *  0.9) ){
			cerr<<".....90%";
		}else if(lc == static_cast<int>(m *  1.0) ){
			cerr<<".....100%";
		}

		removeLeadTrailWS(codestr);	
		removeLeadTrailWS(seqstr);
		if(seqstr.size()==0) continue;
		string alignedseq;
		alignedseq.reserve(3000);
		
		vector<string> * t = new vector<string>;
		t->reserve(3000);
		StringTokenizer(codestr, t);

		/*for(unsigned int i=0; i<t->size(); i++){
			cout<<t->at(i)<<endl;
		}*/
	
		unsigned int k=0;	//index for seqstr
		unsigned int i=0;	//index for t
		
		/* process leading terms containing '-1' */
		while(hasCharacter(t->at(i),'-')){			
			k++;
			i++;
		}
		alignedseq.append(seqstr.substr(0, k));
		alignedseq.append(uniqcode.at(0)->size()-k, '-');

		/* process all other terms */
		for(unsigned int c=0; c<=n; c++){
				
			int ccs;	//parentheis code size
			vector<string> temp;
			temp.reserve(50);
			while(i<t->size() && maxCode(t->at(i))<=c){
				temp.push_back(t->at(i));
				i++;
			}
			if(c==0){
				if(temp.size()==0)		alignedseq.append(1, '-');
				else{				alignedseq.append(1, seqstr[k]); k++;}
				continue;
			}			
		
			ccs = temp.size();
			if(ccs!=0 && !hasCharacter(temp.at(ccs-1), '('))
				ccs = temp.size()-1;
	
			for(unsigned int j=0; j<uniqcode.at(c)->size(); j++){

				if(isInVector(temp, uniqcode.at(c)->at(j))){
					alignedseq.append(1, seqstr[k]);
					k++;					
				}else
					alignedseq.append(1,'-');
			}			
			if(temp.size()==0){
				alignedseq.append(1,'-');
				continue;
			}
			if(ccs==temp.size()-1){
				alignedseq.append(1, seqstr[k]);
				k++;
			}else
				alignedseq.append(1,'-');


		}
		/* write file head */
		/*if(lc==0){
			out<<"begin data;"<<"\n"; 
			out<<"dimensions "<<"\n";
			out<<"ntax ="<<m<<"  nchar ="<<alignedseq.size()<<"\n";
			out<<";"<<"\n";
			out<<"matrix"<<"\n";
		}*/
		/*out<<setiosflags(ios::left)<< setw(12) << label[lc];
		out<<alignedseq<<"\n";
		out<<flush;*/

		out<<">"<<label[lc]<<"\n";
		out<<alignedseq<<"\n";

		delete t;
		lc++;

	} 
	//out<<";"<<"\n";
	//out<<"end;"<<"\n";
	inCode.close();
	inSeq.close();
	
	cerr<<".....100%"<<endl;
	//clock_t tt2 = clock();
	//double duration = (double)(tt2 - tt1) / CLOCKS_PER_SEC;		
	//cout<<"---Total Running time: "<<duration<<" seconds "<<endl;


	return 0;
}


